/***************************************************************************
                          sq_editbase.h  -  description
                             -------------------
    begin                : ??? Mar 3 2005
    copyright            : (C) 2005 by Baryshev Dmitry
    email                : ksquirrel@tut.by
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SQ_EDITBASE_H
#define SQ_EDITBASE_H

#include <qobject.h>
#include <qstringlist.h>

#include <sstream>
#include <csetjmp>

#include <fmt_types.h>
#include <fmt_utils.h>
#include <fileio.h>
#include <fmt_codec_base.h>
#include <error.h>

#include "sq_imageedit.h"

struct SQ_LIBRARY;

class KTempFile;

/*
 *  Base class for all edit tools. It contains all methods needed by
 *  converter, resizer, printer etc. It decodes an image, writes it back to
 *  disk. All specific actions (like colorizing, resizing etc.) are done by
 *  appropriate edit tool.
 */

class SQ_EditBase : public QObject
{
    Q_OBJECT

    public: 
        SQ_EditBase(QObject *parent);
        virtual ~SQ_EditBase();

    protected:

        /*
         *  Do something with decoded image. SQ_Resizer will resize image,
         *  SQ_Contraster will adjust colors etc.
         */
        virtual int manipDecodedImage(fmt_image *im) = 0;

        /*
         *  Create edit tool-specific dialog, and wait for user
         *  input
         */
        virtual void startEditPrivate() = 0;

        /*
         *  Execute specific dialog.
         */
        virtual void dialogReset() = 0;

        /*
         *  Do something with decoded image(manipDecodedImage()) and write it
         *  on disk.
         */
        virtual int manipAndWriteDecodedImage(const QString &name, fmt_image *im);

        /*
         *  Determine next scanline to write on disk. For example, rotate tool
         *  will reimplement this method.
         */
        virtual int determineNextScan(const fmt_image &im, RGBA *scan, int y);

        /*
         *  Reset writing options to default. Convert tool will
         *  reimplement this method to set custom options.
         */
        virtual void initWriteOptions();

        /*
         *  Determine library that should write decoded image.
         */
        virtual void setWritingLibrary();

        /*
         *  Decoding done. Do some cleanup...
         */
        virtual void cycleDone();

        /*
         *  Main decoding loop.
         */
        void decodingCycle();

        /*
         *  Generate preview image for dialog. For example, rotate tool
         *  and filter tool need preview image.
         */
        QImage generatePreview() const;

    private:

        /*
         *  Determine file name. In this file SQ_EditBase will save new image.
         */
        QString adjustFileName(const QString &globalprefix, const QString &name, int replace,
                    QString putto, bool paged = false, int page = 0);

        /*
         *  Copy file :-)
         */
        int copyFile(const QString &src, const QString &dst) const;
        void errorjmp(jmp_buf jmp, const int code);

    signals:
        void convertText(const QString &, bool);
        void oneFileProcessed();
        void done(bool allok);

    protected slots:
        void slotStartEdit();

    protected:
        QStringList         files;
        QString             err_internal, err_failed;
        QString             currentFile;
        int                 error_code, current_page;
        SQ_ImageOptions     imageopt;
        fmt_writeoptions    opt;
        SQ_LIBRARY          *lr, *lw, *altw;
        RGBA                *image;
        QString             special_action, prefix;
        bool                ondisk, multi, last, lastFrame;
        KTempFile           *tempfile;
        fmt_image           *im;
};

#endif
